from typing import TYPE_CHECKING, Any, Dict, List, Type, TypeVar, Union

import attr

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.photo_search import PhotoSearch


T = TypeVar("T", bound="SearchResultsPhoto")


@attr.s(auto_attribs=True)
class SearchResultsPhoto:
    """
    Attributes:
        results (List['PhotoSearch']):
        next_page (Union[Unset, str]):
    """

    results: List["PhotoSearch"]
    next_page: Union[Unset, str] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        results = []
        for results_item_data in self.results:
            results_item = results_item_data.to_dict()

            results.append(results_item)

        next_page = self.next_page

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "results": results,
            }
        )
        if next_page is not UNSET:
            field_dict["next_page"] = next_page

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.photo_search import PhotoSearch

        d = src_dict.copy()
        results = []
        _results = d.pop("results")
        for results_item_data in _results:
            results_item = PhotoSearch.from_dict(results_item_data)

            results.append(results_item)

        next_page = d.pop("next_page", UNSET)

        search_results_photo = cls(
            results=results,
            next_page=next_page,
        )

        search_results_photo.additional_properties = d
        return search_results_photo

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties
